//
// ViewControllerMain.mm
// ======================
// main controller component of matrixProjection
//
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2017-11-01
// UPDATED: 2017-11-10
//
// Copyright (c) 2017 Song Ho Ahn. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

#import "ViewControllerMain.h"
#import "glExtension.h"

@interface ViewControllerMain ()

@end

@implementation ViewControllerMain

@synthesize modelGL;
@synthesize viewGL;
@synthesize radioPerspective;
@synthesize radioOrthographic;
@synthesize radioFill;
@synthesize radioLine;
@synthesize radioPoint;
@synthesize sliderLeft;
@synthesize sliderRight;
@synthesize sliderBottom;
@synthesize sliderTop;
@synthesize sliderNear;
@synthesize sliderFar;
@synthesize labelLeft;
@synthesize labelRight;
@synthesize labelBottom;
@synthesize labelTop;
@synthesize labelNear;
@synthesize labelFar;
@synthesize imageMatrix;

// constants //////////////////////////////////////////////////////////////////
const int   SLIDER_SHIFT = 50;      // center tick
const float SLIDER_SCALE = 10.0f;   // # of ticks per unit



///////////////////////////////////////////////////////////////////////////////
- (void)awakeFromNib
{
    [super awakeFromNib];
    //printf("ViewControllerMain: awakeFromNib\n");
}



///////////////////////////////////////////////////////////////////////////////
- (void)viewDidLoad
{
    [super viewDidLoad];
    //printf("ViewControllerMain: viewDidLoad\n");

    // pass ptr to model
    viewGL.model = &modelGL;

    // init UI controls
    [self resetParameters:nil];
}



///////////////////////////////////////////////////////////////////////////////
- (IBAction)resetParameters:(id)sender
{
    sliderLeft.intValue = (int)(-0.5f * SLIDER_SCALE + SLIDER_SHIFT);
    sliderRight.intValue = (int)(0.5f * SLIDER_SCALE + SLIDER_SHIFT);
    sliderBottom.intValue = (int)(-0.5f * SLIDER_SCALE + SLIDER_SHIFT);
    sliderTop.intValue = (int)(0.5f * SLIDER_SCALE + SLIDER_SHIFT);
    sliderNear.intValue = (int)(1.0f * SLIDER_SCALE);
    sliderFar.intValue = (int)(10.0f * SLIDER_SCALE);

    labelLeft.floatValue = -0.5f;
    labelRight.floatValue = 0.5f;
    labelBottom.floatValue = -0.5f;
    labelTop.floatValue = 0.5f;
    labelNear.floatValue = 1.0f;
    labelFar.floatValue = 10.0f;

    modelGL.setProjection(-0.5f, 0.5f, -0.5f, 0.5f, 1, 10);
    [self updateProjectionMatrix];

    // redraw opengl scene
    [viewGL setNeedsDisplay:YES];
}



///////////////////////////////////////////////////////////////////////////////
- (IBAction)setProjectionMode:(id)sender
{
    if(radioPerspective.state == NSOnState)
    {
        imageMatrix.image = [NSImage imageNamed:@"Perspective"];
        modelGL.setProjectionMode(0);
    }
    else if(radioOrthographic.state == NSOnState)
    {
        imageMatrix.image = [NSImage imageNamed:@"Orthographic"];
        modelGL.setProjectionMode(1);
    }

    [viewGL setNeedsDisplay:YES];
}



///////////////////////////////////////////////////////////////////////////////
- (IBAction)setRenderingMode:(id)sender
{
    if(self.radioFill.state == NSOnState)
    {
        modelGL.setDrawMode(0);
    }
    else if(self.radioLine.state == NSOnState)
    {
        modelGL.setDrawMode(1);
    }
    if(self.radioPoint.state == NSOnState)
    {
        modelGL.setDrawMode(2);
    }

    // need to update opengl state first
    [viewGL.openGLContext makeCurrentContext]; // make RC active before any OpenGL calls
    modelGL.draw();

    [viewGL setNeedsDisplay:YES];
}



///////////////////////////////////////////////////////////////////////////////
- (IBAction)updateParameters:(id)sender
{
    NSSlider* slider = (NSSlider*)sender;
    float value = (slider.floatValue - SLIDER_SHIFT) / SLIDER_SCALE;

    if(slider == sliderLeft)
    {
        labelLeft.stringValue = [NSString stringWithFormat:@"%.1f", value];
        modelGL.setProjectionLeft(value);
    }
    else if(slider == sliderRight)
    {
        labelRight.stringValue = [NSString stringWithFormat:@"%.1f", value];
        modelGL.setProjectionRight(value);
    }
    if(slider == sliderBottom)
    {
        labelBottom.stringValue = [NSString stringWithFormat:@"%.1f", value];
        modelGL.setProjectionBottom(value);
    }
    if(slider == sliderTop)
    {
        labelTop.stringValue = [NSString stringWithFormat:@"%.1f", value];
        modelGL.setProjectionTop(value);
    }
    if(slider == sliderNear)
    {
        labelNear.stringValue = [NSString stringWithFormat:@"%.1f", slider.floatValue / SLIDER_SCALE];
        modelGL.setProjectionNear(slider.floatValue / SLIDER_SCALE);
    }
    if(slider == sliderFar)
    {
        labelFar.stringValue = [NSString stringWithFormat:@"%.1f", slider.floatValue / SLIDER_SCALE];
        modelGL.setProjectionFar(slider.floatValue / SLIDER_SCALE);
    }

    [self updateProjectionMatrix];
    [viewGL setNeedsDisplay:YES];
}



///////////////////////////////////////////////////////////////////////////////
- (void)updateProjectionMatrix
{
    // get matrix as array
    const float* m = modelGL.getProjectionMatrixElements();

    // get label by tag ID starting from 100 to 115
    for(int i = 0, tag = 100; i < 16; ++i, ++tag)
    {
        NSTextField* label = (NSTextField*)[self.view viewWithTag:tag];
        label.stringValue = [NSString stringWithFormat:@"%.2f", m[i]];
    }
}

@end
